@@ -10,6 +10,6 @@ from typing import Any
 def configobj_walker(cfg: Any) -> Any:
     warnings.warn(
         'configobj_walker has moved to ruamel.yaml.util, please update your code',
-        stacklevel=2
+        stacklevel=2,
     )
     return new_configobj_walker(cfg)
